# Copyright (C) 2020 Clariteia SL
#
# This file is part of minos framework.
#
# Minos framework can not be copied and/or distributed without the express
# permission of Clariteia SL.

from abc import (
    ABC,
)

from minos.api_gateway.common import (
    ClientHttp,
    MinosConfig,
)
from minos.api_gateway.discovery.database import (
    MinosRedisClient,
)


class HealthStatusCheck(ABC):
    def __init__(self, config: MinosConfig):
        self.redis_cli = MinosRedisClient(config=config)
        self.redis_conn = self.redis_cli.get_redis_connection()
        self._client_http = ClientHttp()

    async def perform_health_check(self):
        status_code = None
        for key in self.redis_conn.scan_iter():
            try:
                data = self.redis_cli.get_data(key)
                try:
                    status_code = await self.__request(data)
                finally:
                    self.__update(status_code, key, data)
            finally:
                pass

        return True

    async def __request(self, data):
        status_code = None
        if len(data) > 0 and "ip" in data and "name" in data:
            url = "http://{0}/system/health".format(data["ip"])
            if "port" in data:
                if data["port"] is not None:
                    url = "http://{0}:{1}/system/health".format(data["ip"], data["port"])

            try:
                async with ClientHttp() as client:
                    response = await client.get(url=url)

                status_code = response.status
            except Exception:
                print("Error connecting to {0}".format(url))

        return status_code

    def __update(self, status_code, key, data):
        try:
            # If response status code == 200
            # set status to True else,
            # set status to False
            perform_update = False
            if status_code == 200:
                if data["status"] is not True:
                    data["status"] = True
                    perform_update = True
            else:
                if data["status"] is not False:
                    data["status"] = False
                    perform_update = True

            if perform_update:
                self.redis_cli.set_data(key, data)

        finally:
            pass



# Copyright (C) 2020 Clariteia SL
#
# This file is part of minos framework.
#
# Minos framework can not be copied and/or distributed without the express
# permission of Clariteia SL.

import typing as t

from aiohttp import (
    web,
)

from minos.api_gateway.common import (
    MinosConfig,
    RESTService,
)


class DiscoveryService(RESTService):
    def __init__(self, config: MinosConfig, app: web.Application = web.Application(), **kwds: t.Any):
        super().__init__(
            address=config.discovery.connection.host,
            port=config.discovery.connection.port,
            endpoints=config.discovery.endpoints,
            config=config,
            app=app,
            **kwds
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['minos',
 'minos.api_gateway.discovery',
 'minos.api_gateway.discovery.database',
 'minos.api_gateway.discovery.periodic']

package_data = \
{'': ['*']}

install_requires = \
['minos-apigateway-common>=0.0.3,<0.0.4', 'redis>=3.5.3,<4.0.0']

setup_kwargs = {
    'name': 'minos-discovery',
    'version': '0.0.1',
    'description': 'Minos Discovery service for Microservices subscription.',
    'long_description': 'Discovery Service\n=================\n\nMinos API Gateway Discovery Service\n\nCredits\n-------\n\nThis package was created with [Cookiecutter](https://github.com/audreyr/cookiecutter)  and the [Minos Package](https://github.com/Clariteia/minos-pypackage) project template.\n',
    'author': 'Clariteia Devs',
    'author_email': 'devs@clariteia.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://clariteia.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from aiohttp import (
    web,
)

from minos.api_gateway.common import (
    MinosConfig,
)

from .database import (
    MinosRedisClient,
)


async def validate_input(request: web.Request):
    input_json = await request.json()
    errors = True
    if "ip" not in input_json:
        return web.json_response("Value ip not found.", status=400), errors
    if "name" not in input_json:
        return web.json_response("Value name not found.", status=400), errors

    return None, False


async def get_formatted(request: web.Request) -> dict:
    input_json = await request.json()

    if "port" not in input_json:
        input_json["port"] = None

    return {
        "ip": input_json["ip"],
        "port": input_json["port"],
        "name": input_json["name"],
        "status": True,
        "subscribed": True,
    }


class DiscoveryHandlers(object):
    async def discover(self, request: web.Request, config: MinosConfig, **kwargs):
        name = request.query.get("name")
        if not name:
            return web.json_response('Parameter "name" not found.', status=400)
        else:
            # Search by key in Redis and return JSON
            redis_cli = MinosRedisClient(config=config)

            # Get JSON data
            data = redis_cli.get_data(name)
            return web.json_response(data=data)

    async def subscribe(self, request: web.Request, config: MinosConfig, **kwargs):
        validation, errors = await validate_input(request)

        if errors:
            return validation

        input_json = await get_formatted(request)

        redis_cli = MinosRedisClient(config=config)

        redis_cli.set_data(input_json["name"], input_json)

        return web.json_response(text="Service added")

    async def unsubscribe(self, request: web.Request, config: MinosConfig, **kwargs):
        name = request.query.get("name")
        if not name:
            return web.json_response('Parameter "name" not found.', status=400)
        else:
            # Search by key in Redis and set subscription to unsubscribed
            redis_cli = MinosRedisClient(config=config)

            # Get JSON data
            data = redis_cli.get_data(name)

            data["subscribed"] = False

            redis_cli.set_data(name, data)

            return web.json_response(text="Unsubscription done!")

    async def system_health(self, request: web.Request, config: MinosConfig, **kwargs):
        return web.json_response({"host": request.host})



"""
Copyright (C) 2021 Clariteia SL

This file is part of minos framework.

Minos framework can not be copied and/or distributed without the express permission of Clariteia SL.
"""
__version__ = "0.0.1"

from .database import (
    MinosRedisClient,
)
from .periodic import (
    DiscoveryPeriodicHealthChecker,
    HealthStatusCheck,
)
from .service import (
    DiscoveryService,
)
